require 'helper'
require 'adapter_shared_example'
require 'stringio'

describe 'MultiTypograf' do
  context 'adapters' do
    before do
      MultiTypograf.use nil
    end
    context 'when no other implementations are available' do

      CONSTS = [:ArtTypograf, :TypographerHelper, :Typographica, :Typograf]

      before do
        @old_map = MultiTypograf::REQUIREMENT_MAP
        @old_consts = {}
        CONSTS.each do |c|
          @old_consts[c] = Object.const_get c if Object.const_defined?(c)
        end

        MultiTypograf::REQUIREMENT_MAP.each_with_index do |(library, adapter), index|
          MultiTypograf::REQUIREMENT_MAP[index] = ["foo/#{library}", adapter]
        end

        CONSTS.each do |c|
           Object.send :remove_const, c if @old_consts[c]
        end
      end

      after do
        @old_map.each_with_index do |(library, adapter), index|
          MultiTypograf::REQUIREMENT_MAP[index] = [library, adapter]
        end

        CONSTS.each do |c|
           Object.const_set c, @old_consts[c] if @old_consts[c]
        end
      end

      it 'defaults to vendored implemention if no other implementions are available' do
        MultiTypograf.default_adapter.should eq :typograf
      end
    end

    it 'defaults to the best not vendored lib if any other lib available' do
      require 'art_typograf'
      MultiTypograf.adapter.name.should eq 'MultiTypograf::Adapters::ArtTypograf'
    end

    it 'is settable via a symbol' do
      MultiTypograf.use :art_typograf
      MultiTypograf.adapter.name.should eq 'MultiTypograf::Adapters::ArtTypograf'
    end

    it 'is settable via a class' do
      MultiTypograf.use MockDecoder
      MultiTypograf.adapter.name.should eq 'MockDecoder'
    end
  end

  # typographer typographica 
  %w(art_typograf typograf).each do |adapter|
    context adapter do
      it_behaves_like "an adapter", adapter
    end
  end
end

if defined?(MultiTypograf::Filter)
  describe MultiTypograf::Filter do
    it "filter works" do
      pipeline = HTML::Pipeline.new [
        MultiTypograf::Filter
      ]
      pipeline.call(" <p>some text</p>").should eq '<p>some text</p>'
    end
  end
end
